import { makeKMap } from '..';
import { CacheItem } from './cache-item.class';
const value = {};
export class CacheFactory {
    all(model) {
        const modelCache = this.getModelCache(model);
        const res = makeKMap();
        for (const [key, value] of modelCache) {
            res[key] = value.get();
        }
        return res;
    }
    get(model, key) {
        const modelCache = this.getModelCache(model);
        let cacheItem = modelCache.get(key);
        if (undefined === cacheItem) {
            cacheItem = this.createCacheItem(model, undefined, key);
            modelCache.set(key, cacheItem);
        }
        return cacheItem.get();
    }
    put(model, key, value) {
        const modelCache = this.getModelCache(model);
        if (modelCache.has(key)) {
            const cacheItem = modelCache.get(key);
            cacheItem === null || cacheItem === void 0 ? void 0 : cacheItem.put(value);
        }
        else {
            modelCache.set(key, this.createCacheItem(model, value, key));
        }
    }
    publish(model, key, value) {
        const modelCache = this.getModelCache(model);
        if (modelCache.has(key)) {
            const cacheItem = modelCache.get(key);
            cacheItem === null || cacheItem === void 0 ? void 0 : cacheItem.publish(value);
        }
        else {
            const cacheItem = this.createCacheItem(model, value, key);
            modelCache.set(key, cacheItem);
            cacheItem.notifySubscribers();
        }
    }
    has(model, key) {
        const modelCache = this.getModelCache(model);
        return modelCache.has(key);
    }
    subscribe(model, key) {
        const modelCache = this.getModelCache(model);
        if (modelCache.has(key)) {
            const cacheItem = modelCache.get(key);
            return cacheItem.subscribe;
        }
        const cacheItem = this.createCacheItem(model, undefined, key);
        modelCache.set(key, cacheItem);
        return cacheItem.subscribe;
    }
    forget(model, key) {
        var _a;
        const modelCache = this.getModelCache(model);
        (_a = modelCache.get(key)) === null || _a === void 0 ? void 0 : _a.forget();
        modelCache.delete(key);
    }
    createCacheItem(model, value, key) {
        return new CacheItem(value, key, model);
    }
    getModelCache(model) {
        if (model.options.name in value)
            return value[model.options.name];
        value[model.options.name] = new Map();
        return value[model.options.name];
    }
    make(model) {
        const changed = this.createCacheItem(model, '', model.options.name);
        const cache = {
            changed: changed,
            all: () => {
                return this.all(model);
            },
            get: (key, def = undefined) => {
                const res = this.get(model, String(key));
                if (undefined === res)
                    return def;
                return res;
            },
            put: (key, value) => {
                const skey = String(key);
                const res = this.put(model, skey, value);
                return res;
            },
            publish: (key, value) => {
                const skey = String(key);
                const res = this.publish(model, skey, value);
                changed.publish(skey);
                return res;
            },
            has: (key) => {
                const skey = String(key);
                return this.has(model, skey);
            },
            subscribe: (key) => {
                const skey = String(key);
                return this.subscribe(model, skey);
            },
            forget: (key) => {
                const skey = String(key);
                this.forget(model, skey);
            },
        };
        return cache;
    }
}
//# sourceMappingURL=cache-factory.class.js.map