import { SyncAbstractDriverInterface, AsyncAbstractDriverInterface } from '../..';
import { ModelOptionsInterface } from './model-options.interface';
import { DataProxyType } from '../../data/async/data-factory.class';
import { SubscribeClosureInterface, SubscribeInterface, UnsubscribeInterface } from '../../cache/subscribe.interface';
import { KMap } from '../../helpers/make-kurimudb-map.func';
import { CacheInterface } from '../../cache/cache-factory.class';
import { SubscribeConfigInterface } from '../../cache/subscribe-config.interface';
interface AutoSubscribeClosureFunctionInterface {
    (): void | Promise<void>;
}
export declare class BaseModel<DataType extends Record<string, any>, DriverType extends AsyncAbstractDriverInterface | SyncAbstractDriverInterface | undefined = undefined> {
    options: ModelOptionsInterface;
    cache: CacheInterface;
    $: SubscribeInterface;
    data: DataProxyType<DataType>;
    storage: DriverType;
    constructor(options: Partial<ModelOptionsInterface>);
    private checkOptions;
    getItem<Key extends keyof DataType>(key: Key): Promise<DataType[Key] | undefined>;
    bulkGetItem<Key extends keyof DataType>(keys: Array<Key>): Promise<KMap<DataType[Key]>>;
    addItem<Key extends keyof DataType>(key: Key, value: DataType[Key]): Promise<boolean>;
    bulkAddItem<Key extends keyof DataType>(items: Record<Key, DataType[Key]>): Promise<boolean>;
    setItem<Key extends keyof DataType>(key: Key, value: DataType[Key]): Promise<void>;
    bulkSetItem<Key extends keyof DataType>(items: Record<Key, DataType[Key]>): Promise<boolean>;
    hasItem<Key extends keyof DataType>(key: Key): Promise<boolean>;
    removeItem<Key extends keyof DataType>(key: Key): Promise<void>;
    bulkRemoveItem<Key extends keyof DataType>(keys: Array<Key>): Promise<boolean>;
    subscribeItem<Key extends keyof DataType>(key: Key, closFunc: SubscribeClosureInterface, config?: SubscribeConfigInterface): UnsubscribeInterface;
    getSubscribeItemFunc<Key extends keyof DataType>(key: Key): SubscribeInterface;
    clone<T>(value: T): Promise<T>;
    auto$(subClosFunc: AutoSubscribeClosureFunctionInterface, config?: SubscribeConfigInterface): UnsubscribeInterface | Promise<UnsubscribeInterface>;
    stream$<Key extends keyof DataType>(getData: () => Array<Key> | Record<Key, DataType[Key]> | Promise<Array<Key> | Record<Key, DataType[Key]>> | Array<string> | Record<string, any> | Promise<Array<string> | Record<Key, any>>, handler: (key: Key, value: DataType[Key]) => void | unknown | Promise<void | unknown>, config?: SubscribeConfigInterface & {
        autoLoad?: boolean;
    }): {
        load: () => Promise<void>;
        startAutoLoad: () => void;
        stopAutoLoad: () => void;
        unsubscribe: () => void;
        getKeys: () => IterableIterator<Key>;
        getLength: () => number;
    };
}
export {};
